@@ -1,4 +1,5 @@
 import abc
+import errno
 import os
 import re
 import sys
@@ -14,7 +15,10 @@ import whoosh.qparser
 from whoosh.filedb.filestore import RamStorage
 
 from flask import current_app
-from flask_sqlalchemy import BaseQuery
+try:
+    from flask_sqlalchemy.query import Query
+except ImportError:
+    from flask_sqlalchemy import BaseQuery as Query
 from sqlalchemy import text, event
 from sqlalchemy.inspection import inspect
 from sqlalchemy.orm.mapper import Mapper
@@ -31,7 +35,7 @@ UPDATE_KWD = 'update'
 DELETE_KWD = 'delete'
 
 
-__version__ = '0.7.0'
+__version__ = '0.9.1'
 
 
 def _get_app(obj):
@@ -40,8 +44,14 @@ def _get_app(obj):
 def _get_config(obj):
     return _get_app(obj).extensions['whooshee']
 
+def _assure_dirs_exists(path):
+    try:
+        os.makedirs(path)
+    except OSError as err:
+        if err.errno != errno.EEXIST:
+            raise
 
-class WhoosheeQuery(BaseQuery):
+class WhoosheeQuery(Query):
     """An override for SQLAlchemy query used to do fulltext search."""
 
     def whooshee_search(self, search_string, group=whoosh.qparser.OrGroup, whoosheer=None,
@@ -119,11 +129,11 @@ class WhoosheeQuery(BaseQuery):
 
         if order_by_relevance < 0: # we want all returned rows ordered
             search_query = search_query.order_by(sqlalchemy.sql.expression.case(
-                [(attr == uniq_val, index) for index, uniq_val in enumerate(res)],
+                *[(attr == uniq_val, index) for index, uniq_val in enumerate(res)],
             ))
         elif order_by_relevance > 0: # we want only number of specified rows ordered
             search_query = search_query.order_by(sqlalchemy.sql.expression.case(
-                [(attr == uniq_val, index) for index, uniq_val in enumerate(res) if index < order_by_relevance],
+                *[(attr == uniq_val, index) for index, uniq_val in enumerate(res) if index < order_by_relevance],
                 else_=order_by_relevance
             ))
         else: # no ordering
@@ -262,8 +272,8 @@ class Whooshee(object):
             warnings.warn(WhoosheeDeprecationWarning("The config key WHOOSHE_MIN_STRING_LEN has been renamed to WHOOSHEE_MIN_STRING_LEN. The mispelled config key is deprecated and will be removed in upcoming releases. Change it to WHOOSHEE_MIN_STRING_LEN to suppress this warning"))
             config['search_string_min_len'] = app.config.get('WHOOSHE_MIN_STRING_LEN')
 
-        if not os.path.exists(config['index_path_root']):
-            os.makedirs(config['index_path_root'])
+        _assure_dirs_exists(config['index_path_root'])
+
 
     def register_whoosheer(self, wh):
         """This will register the given whoosher on `whoosheers`, create the
@@ -286,7 +296,7 @@ class Whooshee(object):
                     pass
 
                 # ensure there can be a stable MRO
-                elif query_class not in (BaseQuery, SQLAQuery, WhoosheeQuery):
+                elif query_class not in (Query, SQLAQuery, WhoosheeQuery):
                     query_class_name = query_class.__name__
                     model.query_class = type(
                         "Whooshee{}".format(query_class_name), (query_class, self.query), {}
@@ -406,8 +416,7 @@ class Whooshee(object):
             if whoosh.index.exists_in(index_path):
                 index = whoosh.index.open_dir(index_path)
             else:
-                if not os.path.exists(index_path):
-                    os.makedirs(index_path)
+                _assure_dirs_exists(index_path)
                 index = whoosh.index.create_in(index_path, wh.schema)
             return index
 
