@@ -5,12 +5,17 @@ import tempfile
 from unittest import TestCase
 import string
 
-import flexmock
+from flexmock import flexmock
 import whoosh
 from whoosh.filedb.filestore import RamStorage
 from flask import Flask
-from flask_sqlalchemy import SQLAlchemy, BaseQuery
+from flask_sqlalchemy import SQLAlchemy
+try:
+    from flask_sqlalchemy.query import Query
+except ImportError:
+    from flask_sqlalchemy import BaseQuery as Query
 from sqlalchemy.orm import Query as SQLAQuery
+from sqlalchemy.sql import text
 from flask_whooshee import AbstractWhoosheer, Whooshee, WhoosheeQuery
 
 
@@ -31,6 +36,9 @@ class BaseTestCases(object):
 
         def setUp(self):
 
+            self.ctx = self.app.app_context()
+            self.ctx.push()
+
             class User(self.db.Model):
                 id = self.db.Column(self.db.Integer, primary_key=True)
                 name = self.db.Column(self.db.String)
@@ -119,6 +127,7 @@ class BaseTestCases(object):
             shutil.rmtree(self.app.config['WHOOSHEE_DIR'], ignore_errors=True)
             Whooshee.whoosheers = []
             self.db.drop_all()
+            self.ctx.pop()
 
         # tests testing model whoosheers should have mw in their name, for custom whoosheers it's cw
         # ideally, there should be a separate class for model whoosheer and custom whoosheer
@@ -263,7 +272,7 @@ class BaseTestCases(object):
             # generall reindex
             self.wh.reindex()
             # put stallone directly in db and find him only after reindex
-            result = self.db.session.execute("INSERT INTO entry VALUES (100, 'rambo', 'pack of one two and three', {0})".format(self.u3.id))
+            result = self.db.session.execute(text("INSERT INTO entry VALUES (100, 'rambo', 'pack of one two and three', {0})".format(self.u3.id)))
             self.db.session.commit()
             found = self.Entry.query.join(self.User).whooshee_search('rambo').all()
             self.assertEqual(len(found), 0)
@@ -299,7 +308,12 @@ class BaseTestCases(object):
             found = self.Entry.query.whooshee_search('blah blah blah').all()
             self.assertEqual(len(found), 0)
 
-            expectation.reset()
+            # Compatibility with old flexmock for Python 2
+            if hasattr(expectation, "reset"):
+                expectation.reset()
+            else:
+                expectation._reset()
+
             # now let's try writing without the exception to verify that lock was released
             self.db.session.add(self.e1)
             self.db.session.commit()
@@ -340,7 +354,7 @@ class BaseTestCases(object):
             self.assertEqual(len(found), 0)
 
             # make sure that the entry has actually been deleted from the whoosh index
-            # https://github.com/bkabrda/flask-whooshee/pull/26#issuecomment-257549715
+            # https://github.com/fedora-copr/flask-whooshee/pull/26#issuecomment-257549715
             whoosheer = next(w for w in self.wh.whoosheers if set(w.models) == set([self.Entry]))
             self.assertEqual(len(whoosheer.search('blah blah blah')), 0)
 
@@ -407,13 +421,6 @@ class TestsWithInitApp(BaseTestCases.BaseTest):
         # intialization is possible
         self.wh.init_app(self.app)
 
-        self.ctx = self.app.app_context()
-        self.ctx.push()
-
-    def tearDown(self):
-        super(TestsWithInitApp, self).tearDown()
-        self.ctx.pop()
-
 
 class TestsAppWithMemoryStorage(TestCase):
 
@@ -428,6 +435,9 @@ class TestsAppWithMemoryStorage(TestCase):
         self.db = SQLAlchemy(self.app)
         self.wh = Whooshee(self.app)
 
+        self.ctx = self.app.app_context()
+        self.ctx.push()
+
         class User(self.db.Model):
             id = self.db.Column(self.db.Integer, primary_key=True)
             name = self.db.Column(self.db.String)
@@ -491,7 +501,7 @@ class TestsAppWithMemoryStorage(TestCase):
         self.Entry = Entry
         self.EntryUserWhoosheer = EntryUserWhoosheer
 
-        self.db.create_all(app=self.app)
+        self.db.create_all()
 
         self.u1 = User(name=u'chuck')
         self.u2 = User(name=u'arnold')
@@ -507,7 +517,8 @@ class TestsAppWithMemoryStorage(TestCase):
         self.db.session.commit()
 
     def tearDown(self):
-        self.db.drop_all(app=self.app)
+        self.db.drop_all()
+        self.ctx.pop()
 
     def test_memory_storage(self):
         indexes = self.app.extensions['whooshee']['whoosheers_indexes']
@@ -515,6 +526,7 @@ class TestsAppWithMemoryStorage(TestCase):
 
 
 class TestBigInteger(TestCase):
+    # pylint: disable=too-many-instance-attributes
 
     def setUp(self):
         self.app = Flask(__name__)
@@ -527,6 +539,9 @@ class TestBigInteger(TestCase):
         self.db = SQLAlchemy(self.app)
         self.wh = Whooshee(self.app)
 
+        self.ctx = self.app.app_context()
+        self.ctx.push()
+
         class User(self.db.Model):
             id = self.db.Column(self.db.Integer, primary_key=True)
             name = self.db.Column(self.db.String)
@@ -543,7 +558,7 @@ class TestBigInteger(TestCase):
         self.User = User
         self.Entry = Entry
 
-        self.db.create_all(app=self.app)
+        self.db.create_all()
 
         self.u1 = User(name=u'chuck')
         self.e1 = Entry(id=1000000000000, title=u'chuck nr. 1 article', content=u'blah blah blah', user=self.u1)
@@ -551,7 +566,8 @@ class TestBigInteger(TestCase):
         self.db.session.commit()
 
     def tearDown(self):
-        self.db.drop_all(app=self.app)
+        self.db.drop_all()
+        self.ctx.pop()
 
     def test_add(self):
             # test that the add operation works for Big Integer PK
@@ -641,8 +657,11 @@ class TestMultipleApps(TestCase):
         self.Entry = Entry
         self.EntryUserWhoosheer = EntryUserWhoosheer
 
-        self.db.create_all(app=self.app1)
-        self.db.create_all(app=self.app2)
+        with self.app1.app_context():
+            self.db.create_all()
+
+        with self.app2.app_context():
+            self.db.create_all()
 
         self.u1 = User(name=u'chuck')
         self.u2 = User(name=u'arnold')
@@ -658,8 +677,10 @@ class TestMultipleApps(TestCase):
     def tearDown(self):
         shutil.rmtree(self.app1.config['WHOOSHEE_DIR'], ignore_errors=True)
         shutil.rmtree(self.app2.config['WHOOSHEE_DIR'], ignore_errors=True)
-        self.db.drop_all(app=self.app1)
-        self.db.drop_all(app=self.app2)
+        with self.app1.app_context():
+            self.db.drop_all()
+        with self.app2.app_context():
+            self.db.drop_all()
 
     def test_multiple_apps(self):
         # IIUC, you can't add the same model instance under multiple apps with flask-sqlalchemy
@@ -707,14 +728,14 @@ class TestDoesntMixesWithModelQueryClass(TestCase):
         self.wh = Whooshee(self.app)
 
     def test_mixes_with_model_query(self):
-        class CustomQueryClass(BaseQuery):
+        class CustomQueryClass(Query):
             pass
 
         self._make_model_and_whoosheer(CustomQueryClass)
         self.assertEqual('WhoosheeCustomQueryClass', self.user_model.query_class.__name__)
 
     def test_doesnt_mix_with_default_query_class(self):
-        self._make_model_and_whoosheer(BaseQuery)
+        self._make_model_and_whoosheer(Query)
         self.assertIs(self.wh.query, self.user_model.query_class)
 
     def test_doesnt_mix_with_explicit_whooshee_query_class(self):
